@@ -1,7 +1,19 @@
 /* PDCurses */
 
-#ifndef __PDC_WINCON_WIN_H__
-#define __PDC_WINCON_WIN_H__
+#ifndef __PDC_WIN_H__
+#define __PDC_WIN_H__
+
+#if defined(_MSC_VER) && !defined(_CRT_SECURE_NO_DEPRECATE)
+# define _CRT_SECURE_NO_DEPRECATE 1   /* kill nonsense warnings */
+#endif
+
+#if defined(_MSC_VER) && !defined(_CRT_SECURE_NO_DEPRECATE)
+# define _CRT_SECURE_NO_DEPRECATE 1   /* kill nonsense warnings */
+#endif
+
+#if defined( PDC_FORCE_UTF8) && !defined( PDC_WIDE)
+   #define PDC_WIDE
+#endif
 
 #if defined(PDC_WIDE) && !defined(UNICODE)
 # define UNICODE
@@ -12,10 +24,6 @@
 #undef MOUSE_MOVED
 #include <curspriv.h>
 
-#if defined(_MSC_VER) && !defined(_CRT_SECURE_NO_DEPRECATE)
-# define _CRT_SECURE_NO_DEPRECATE 1   /* kill nonsense warnings */
-#endif
-
 typedef struct {short r, g, b; bool mapped;} PDCCOLOR;
 
 extern PDCCOLOR pdc_color[PDC_MAXCOL];
@@ -25,7 +33,7 @@ extern DWORD pdc_quick_edit;
 extern DWORD pdc_last_blink;
 extern short pdc_curstoreal[16], pdc_curstoansi[16];
 extern short pdc_oldf, pdc_oldb, pdc_oldu;
-extern bool pdc_conemu, pdc_ansi;
+extern bool pdc_conemu, pdc_wt, pdc_ansi;
 
 extern void PDC_blink_text(void);
 
