@@ -204,7 +204,7 @@ function address at runtime. if the method isn't available, that means we're on
 and older operating system, so we just return the original value */
 static LONG scale_font_for_current_dpi( LONG size)
 {
-    typedef LONG(__stdcall *GetDpiForSystemProc)();
+    typedef LONG(STDAPICALLTYPE *GetDpiForSystemProc)();
     HMODULE user32Dll = LoadLibrary( _T("User32.dll"));
 
     if ( user32Dll)
@@ -235,7 +235,7 @@ their preferred one. If that font fails to load, we will fallback
 to the global default (currently "Courier New") */
 int PDC_set_preferred_fontface( const TCHAR* fontface)
 {
-    int len = fontface == 0 ? 0 : wcslen( fontface);
+    int len = fontface == 0 ? 0 : (int) wcslen( fontface);
     if ( len < sizeof( PDC_preferred_fontface))
     {
         wcsncpy( PDC_preferred_fontface, fontface, len);
@@ -270,19 +270,18 @@ static int PDC_fontface_exists( const TCHAR* fontface)
 
 static LOGFONT PDC_get_logical_font( const int font_idx)
 {
-    if ( PDC_font_size < 0)
-    {
-        PDC_font_size = scale_font_for_current_dpi( 15); /* default 15 points */
-    }
-
     /* see if the user has overridden the default fontface. */
     if ( wcslen( PDC_font_name) == 0 && PDC_fontface_exists(PDC_preferred_fontface))
     {
         wcsncpy( PDC_font_name, PDC_preferred_fontface, sizeof(PDC_font_name));
     }
 
-    LOGFONT lf;
+    if ( PDC_font_size < 0)
+    {
+        PDC_font_size = scale_font_for_current_dpi( 12); /* default 12 points */
+    }
 
+    LOGFONT lf;
     memset(&lf, 0, sizeof(LOGFONT));        /* Clear out structure. */
     lf.lfHeight = -PDC_font_size;
 #ifdef PDC_WIDE
@@ -335,15 +334,16 @@ int PDC_choose_a_new_font( void)
     cf.hwndOwner = PDC_hWnd;
     cf.lpLogFont = &lf;
     rval = ChooseFont( &cf);
-    if( rval)
+    if( rval) {
 #ifdef PDC_WIDE
         wcscpy( PDC_font_name, lf.lfFaceName);
 #else
         strcpy( PDC_font_name, lf.lfFaceName);
 #endif
+        PDC_font_size = -lf.lfHeight;
+        debug_printf( "output size: %d\n", lf.lfHeight);
+    }
     debug_printf( "rval %d; %ld\n", rval, CommDlgExtendedError( ));
-    debug_printf( "output size: %d\n", lf.lfHeight);
-    PDC_font_size = -lf.lfHeight;
     return( rval);
 }
 
@@ -371,7 +371,7 @@ static bool character_is_in_font( chtype ichar)
     int i;
     WCRANGE *wptr = PDC_unicode_range_data->ranges;
 
-    if( (ichar & A_ALTCHARSET) && (ichar & A_CHARTEXT) < 0x80)
+    if( _is_altcharset( ichar))
        ichar = acs_map[ichar & 0x7f];
     ichar &= A_CHARTEXT;
     if( ichar > MAX_UNICODE)  /* assume combining chars won't be */
@@ -468,7 +468,7 @@ void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
 
     while( len)
     {
-        const attr_t attrib = (attr_t)( *srcp >> PDC_REAL_ATTR_SHIFT);
+        const attr_t attrib = (attr_t)( *srcp & ~A_CHARTEXT);
         const int color = (int)(( *srcp & A_COLOR) >> PDC_COLOR_SHIFT);
         attr_t new_font_attrib = (*srcp & (A_BOLD | A_ITALIC));
         RECT clip_rect;
@@ -484,7 +484,7 @@ void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
                   && (in_font == character_is_in_font( srcp[i])
                               || (srcp[i] & A_CHARTEXT) == MAX_UNICODE)
 #endif
-                  && attrib == (attr_t)( srcp[i] >> PDC_REAL_ATTR_SHIFT); i++)
+                  && attrib == (attr_t)( srcp[i] & ~A_CHARTEXT); i++)
         {
             chtype ch = srcp[i] & A_CHARTEXT;
 
@@ -520,7 +520,7 @@ void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
                 }
             }
 #endif
-            if( (srcp[i] & A_ALTCHARSET) && ch < 0x80)
+            if( _is_altcharset( srcp[i]))
                 ch = acs_map[ch & 0x7f];
             else if( ch < 32)
                ch = starting_ascii_to_unicode[ch];
@@ -533,7 +533,6 @@ void PDC_transform_line_given_hdc( const HDC hdc, const int lineno,
                mbtowc( &z, &c, 1);
                ch = (chtype)z;
             }
-            assert( "We should never get here");
 #endif
             buff[olen] = (wchar_t)ch;
             lpDx[olen] = PDC_cxChar;
