@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -78,6 +78,7 @@ using Editor = PlaybackService::Editor;
 #define MESSAGE_RELOAD_OUTPUT 1010
 #define MESSAGE_LOAD_PLAYBACK_CONTEXT 1011
 #define MESSAGE_MARK_TRACK_PLAYED 1012
+#define MESSAGE_NEXT_TRACK_EDITED 1013
 
 class StreamMessage: public Message {
     public:
@@ -308,6 +309,7 @@ void PlaybackService::ProcessMessage(IMessage &message) {
         this->InitRemotes();
     }
     else if (type == MESSAGE_MARK_TRACK_PLAYED) {
+        lastfm::Scrobble(this->playingTrack);
         this->MarkTrackAsPlayed(message.UserData1()); /* UserData1 is a trackId */
     }
     else if (type == MESSAGE_STREAM_EVENT) {
@@ -391,6 +393,21 @@ void PlaybackService::ProcessMessage(IMessage &message) {
 
         this->PlaybackStateChanged(static_cast<PlaybackState>(eventType));
     }
+    else if (type == MESSAGE_NEXT_TRACK_EDITED) {
+        if (transport->GetPlaybackState() != PlaybackState::Stopped) {
+            const size_t updatedIndex = static_cast<size_t>(message.UserData1());
+
+            if (updatedIndex != NO_POSITION) {
+                this->index = updatedIndex;
+                this->nextIndex = NO_POSITION; /* force recalc */
+            }
+
+            /* the user may be moving things around aggressively; we wait a few seconds
+            before actually scheduling the next track lookup to avoid a bunch of unnecessary
+            I/O (note: the I/O may be over the network in some cases) */
+            this->messageQueue.Debounce(Message::Create(this, MESSAGE_PREPARE_NEXT_TRACK, updatedIndex), 2500);
+        }
+    }
     else if (type == MESSAGE_PREPARE_NEXT_TRACK) {
         if (transport->GetPlaybackState() != PlaybackState::Stopped) {
             const size_t updatedIndex = static_cast<size_t>(message.UserData1());
@@ -502,19 +519,17 @@ void PlaybackService::OnTrackChanged(size_t pos, TrackPtr track) {
     this->messageQueue.Remove(this, MESSAGE_MARK_TRACK_PLAYED);
 
     if (track && this->transport->GetStreamState() == StreamState::Playing) {
-        /* TODO: maybe consider folding Scrobble() the `MarkTrackAsPlayed` logic?
-        needs a bit more thought */
-        lastfm::Scrobble(track);
-
         /* we consider a track to be played if (1) it enters the playing state and
         it's less than 10 seconds long, or (2) it enters the playing state, and
-        remains playing for > 10 seconds */
+        remains playing for > 25% of its duration seconds */
         const double duration = this->transport->GetDuration();
         if (duration > 0 && duration < 10.0) {
+            lastfm::Scrobble(track);
             this->MarkTrackAsPlayed(track->GetId());
         }
         else {
-            POST_DELAYED(this, MESSAGE_MARK_TRACK_PLAYED, track->GetId(), 0, 10000LL);
+            const int64_t delay = (int64_t)(duration * 0.25f) * 1000LL;
+            POST_DELAYED(this, MESSAGE_MARK_TRACK_PLAYED, track->GetId(), 0, delay);
         }
     }
 
@@ -1001,7 +1016,7 @@ PlaybackService::Editor::~Editor() {
             }
 
             this->queue.Post(Message::Create(
-                &this->playback, MESSAGE_PREPARE_NEXT_TRACK, this->playIndex, 0));
+                &this->playback, MESSAGE_NEXT_TRACK_EDITED, this->playIndex));
         }
 
         this->playback.messageQueue.Post(Message::Create(
@@ -1161,4 +1176,4 @@ ITransport::Gain PlaybackService::GainAtIndex(size_t index) {
     }
 
     return result;
-}
\ No newline at end of file
+}
