@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -498,7 +498,7 @@ mcsdk_export void mcsdk_svc_playback_reload_output(mcsdk_svc_playback pb) {
     PLAYBACK(pb)->ReloadOutput();
 }
 
-mcsdk_export mcsdk_track_list mcsdk_svc_playback_clone(mcsdk_svc_playback pb) {
+mcsdk_export mcsdk_track_list mcsdk_svc_playback_clone_track_list(mcsdk_svc_playback pb) {
     return mcsdk_track_list { PLAYBACK(pb)->Clone() };
 }
 
@@ -993,7 +993,7 @@ mcsdk_export double mcsdk_audio_stream_get_duration(mcsdk_audio_stream as) {
 }
 
 mcsdk_export bool mcsdk_audio_stream_open_uri(mcsdk_audio_stream as, const char* uri) {
-    return AUDIOSTREAM(as)->OpenStream(uri);
+    return AUDIOSTREAM(as)->OpenStream(uri, nullptr); /* TODO: fixme; should pass output if available */
 }
 
 mcsdk_export void mcsdk_audio_stream_interrupt(mcsdk_audio_stream as) {
@@ -1035,11 +1035,11 @@ class mcsdk_audio_player_callback_proxy: public Player::EventListener {
                 }
             }
         }
-        virtual void OnPlayerAlmostEnded(Player *player) {
+        virtual void OnPlayerStreamEof(Player *player) {
             std::unique_lock<std::mutex> lock(this->context->event_mutex);
             for (auto c : callbacks) {
-                if (c->on_almost_ended) {
-                    c->on_almost_ended(mcsdk_audio_player { context });
+                if (c->on_stream_eof) {
+                    c->on_stream_eof(mcsdk_audio_player { context });
                 }
             }
         }
@@ -1055,18 +1055,18 @@ class mcsdk_audio_player_callback_proxy: public Player::EventListener {
             std::unique_lock<std::mutex> lock(this->context->event_mutex);
             for (auto c : callbacks) {
                 if (c->on_error) {
-                    c->on_error(mcsdk_audio_player { context });
+                    c->on_error(mcsdk_audio_player { context }, mcsdk_playback_error_open_failed);
                 }
             }
         }
         virtual void OnPlayerDestroying(Player *player) {
             std::unique_lock<std::mutex> lock(this->context->event_mutex);
+            this->context->player_finished = true;
             for (auto c : callbacks) {
                 if (c->on_destroying) {
                     c->on_destroying(mcsdk_audio_player { context });
                 }
             }
-            this->context->player_finished = true;
             this->context->finished_condition.notify_all();
         }
         virtual void OnPlayerMixPoint(Player *player, int id, double time) {
@@ -1243,7 +1243,7 @@ mcsdk_export void mcsdk_svc_indexer_schedule(mcsdk_svc_indexer in, mcsdk_svc_ind
 }
 
 mcsdk_export void mcsdk_svc_indexer_stop(mcsdk_svc_indexer in) {
-    INDEXER(in)->Stop();
+    INDEXER(in)->Shutdown();
 }
 
 mcsdk_export mcsdk_svc_indexer_state mcsdk_svc_indexer_get_state(mcsdk_svc_indexer in) {
