@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -42,18 +42,22 @@
 #include <musikcore/library/Indexer.h>
 #include <musikcore/runtime/Message.h>
 #include <musikcore/debug.h>
+
 #include <limits>
+#include <filesystem>
 
 static const std::string TAG = "LocalLibrary";
 static bool scheduleSyncDueToDbUpgrade = false;
 
+namespace fs = std::filesystem;
+
 using namespace musik::core;
 using namespace musik::core::library;
 using namespace musik::core::runtime;
 using namespace std::chrono;
 
-#define DATABASE_VERSION 9
-#define VERBOSE_LOGGING 0
+#define DATABASE_VERSION 10
+#define VERBOSE_LOGGING 1
 #define MESSAGE_QUERY_COMPLETED 5000
 
 class LocalResourceLocator: public ILibrary::IResourceLocator {
@@ -167,12 +171,12 @@ std::string LocalLibrary::GetLibraryDirectory() {
         directory.append(this->identifier + "/" );
     }
 
-    boost::filesystem::path dir(directory);
-    if(!boost::filesystem::exists(dir)){
-        boost::filesystem::create_directories(dir);
+    fs::path dir(fs::u8path(directory));
+    if(!fs::exists(dir)){
+        fs::create_directories(dir);
     }
 
-    directory = dir.string();
+    directory = dir.u8string();
 
     return directory;
 }
@@ -405,6 +409,10 @@ static void upgradeV8ToV9(db::Connection& db) {
     db.Execute("ALTER TABLE tracks ADD COLUMN date_updated REAL DEFAULT null");
 }
 
+static void upgradeV9ToV10(db::Connection& db) {
+    db.Execute("UPDATE tracks set disc=1 where disc is null or disc like \"\"");
+}
+
 static void setVersion(db::Connection& db, int version) {
     db.Execute("DELETE FROM version");
     db::Statement stmt("INSERT INTO version VALUES(?)", db);
@@ -650,6 +658,10 @@ void LocalLibrary::CreateDatabase(db::Connection &db){
         upgradeV8ToV9(db);
     }
 
+    if (lastVersion >= 1 && lastVersion < 10) {
+        upgradeV9ToV10(db);
+    }
+
     /* ensure our version is set correctly */
     setVersion(db, DATABASE_VERSION);
 
