@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -45,10 +45,13 @@
 #include <musikcore/io/DataStreamFactory.h>
 
 #include <unordered_map>
+#include <chrono>
 
 using namespace musik::core;
 using namespace musik::core::sdk;
 
+using namespace std::chrono;
+
 #define GENRE_TRACK_COLUMN_NAME "genre"
 #define GENRES_TABLE_NAME "genres"
 #define GENRE_TRACK_JUNCTION_TABLE_NAME "track_genres"
@@ -153,7 +156,7 @@ double IndexerTrack::GetDouble(const char* key, double defaultValue) {
 }
 
 void IndexerTrack::SetValue(const char* metakey, const char* value) {
-    if (metakey && value) {
+    if (metakey && value && strlen(value)) {
         this->internalMetadata->metadata.insert(
             std::pair<std::string, std::string>(metakey,value));
     }
@@ -255,20 +258,21 @@ int64_t IndexerTrack::GetId() {
 }
 
 bool IndexerTrack::NeedsToBeIndexed(
-    const boost::filesystem::path &file,
+    const std::filesystem::path &file,
     db::Connection &dbConnection)
 {
     try {
-        this->SetValue("path", file.string().c_str());
-        this->SetValue("filename", file.string().c_str());
+        this->SetValue("path", file.u8string().c_str());
+        this->SetValue("filename", file.u8string().c_str());
 
-        size_t lastDot = file.leaf().string().find_last_of(".");
+        size_t lastDot = file.filename().u8string().find_last_of(".");
         if (lastDot != std::string::npos) {
-            this->SetValue("extension", file.leaf().string().substr(lastDot + 1).c_str());
+            this->SetValue("extension", file.filename().u8string().substr(lastDot + 1).c_str());
         }
 
-        const size_t fileSize = (size_t) boost::filesystem::file_size(file);
-        const size_t fileTime = (size_t) boost::filesystem::last_write_time(file);
+        const size_t fileSize = (size_t) std::filesystem::file_size(file);
+        const int64_t fileTime = (int64_t) duration_cast<milliseconds>(
+            std::filesystem::last_write_time(file).time_since_epoch()).count();
 
         this->SetValue("filesize", std::to_string(fileSize).c_str());
         this->SetValue("filetime", std::to_string(fileTime).c_str());
@@ -285,7 +289,7 @@ bool IndexerTrack::NeedsToBeIndexed(
         if (stmt.Step() == db::Row) {
             this->trackId = stmt.ColumnInt64(0);
             const int dbFileSize = stmt.ColumnInt32(2);
-            const int dbFileTime = stmt.ColumnInt32(3);
+            const int64_t dbFileTime = stmt.ColumnInt64(3);
 
             if (fileSize == dbFileSize && fileTime == dbFileTime) {
                 return false;
@@ -298,6 +302,17 @@ bool IndexerTrack::NeedsToBeIndexed(
     return true;
 }
 
+static int stringToInt(const std::string& str, const int defaultValue) {
+    try {
+        if (str.size()) {
+            return std::stoi(str, 0, 10);
+        }
+    }
+    catch (...) {
+    }
+    return defaultValue;
+}
+
 static int64_t writeToTracksTable(
     db::Connection &dbConnection,
     IndexerTrack& track)
@@ -330,33 +345,35 @@ static int64_t writeToTracksTable(
         query =
             "UPDATE tracks "
             "SET track=?, disc=?, bpm=?, duration=?, filesize=?, "
-            "    title=?, filename=?, filetime=?, path_id=?, "
+            "    title=?, rating=?, filename=?, filetime=?, path_id=?, "
             "    date_updated=julianday('now'), external_id=? "
             "WHERE id=?";
     }
     else {
         query =
             "INSERT INTO tracks "
-            "(track, disc, bpm, duration, filesize, title, filename, "
+            "(track, disc, bpm, duration, filesize, title, rating, filename, "
             " filetime, path_id, external_id, date_added, date_updated) "
-            "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, julianday('now'), julianday('now'))";
+            "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, julianday('now'), julianday('now'))";
     }
 
     db::Statement stmt(query.c_str(), dbConnection);
 
-    stmt.BindText(0, track.GetString("track"));
-    stmt.BindText(1, track.GetString("disc"));
-    stmt.BindText(2, track.GetString("bpm"));
-    stmt.BindInt32(3, track.GetInt32("duration"));
-    stmt.BindInt32(4, track.GetInt32("filesize"));
-    stmt.BindText(5, track.GetString("title"));
-    stmt.BindText(6, track.GetString("filename"));
-    stmt.BindInt32(7, track.GetInt32("filetime"));
-    stmt.BindInt64(8, track.GetInt64("path_id"));
-    stmt.BindText(9, track.GetString("external_id"));
+    int bindPos = 0;
+    stmt.BindInt32(bindPos++, stringToInt(track.GetString("track"), 1));
+    stmt.BindInt32(bindPos++, stringToInt(track.GetString("disc"), 1));
+    stmt.BindText(bindPos++, track.GetString("bpm"));
+    stmt.BindInt32(bindPos++, track.GetInt32("duration"));
+    stmt.BindInt32(bindPos++, track.GetInt32("filesize"));
+    stmt.BindText(bindPos++, track.GetString("title"));
+    stmt.BindInt32(bindPos++, stringToInt(track.GetString("rating"), 0));
+    stmt.BindText(bindPos++, track.GetString("filename"));
+    stmt.BindInt64(bindPos++, track.GetInt64("filetime"));
+    stmt.BindInt64(bindPos++, track.GetInt64("path_id"));
+    stmt.BindText(bindPos++, track.GetString("external_id"));
 
     if (id != 0) {
-        stmt.BindInt64(10, id);
+        stmt.BindInt64(bindPos++, id);
     }
 
     if (stmt.Step() == db::Done) {
@@ -380,23 +397,24 @@ static void removeRelation(
 }
 
 static void removeKnownFields(Track::MetadataMap& metadata) {
-    metadata.erase("track");
-    metadata.erase("disc");
-    metadata.erase("bpm");
-    metadata.erase("duration");
-    metadata.erase("title");
-    metadata.erase("filename");
-    metadata.erase("filetime");
-    metadata.erase("filesize");
-    metadata.erase("title");
-    metadata.erase("path");
-    metadata.erase("extension");
-    metadata.erase("genre");
-    metadata.erase("artist");
     metadata.erase("album_artist");
     metadata.erase("album");
-    metadata.erase("source_id");
+    metadata.erase("artist");
+    metadata.erase("bpm");
+    metadata.erase("disc");
+    metadata.erase("duration");
+    metadata.erase("extension");
     metadata.erase("external_id");
+    metadata.erase("filename");
+    metadata.erase("filesize");
+    metadata.erase("filetime");
+    metadata.erase("genre");
+    metadata.erase("path");
+    metadata.erase("rating");
+    metadata.erase("source_id");
+    metadata.erase("title");
+    metadata.erase("title");
+    metadata.erase("track");
     metadata.erase("visible");
 }
 
@@ -748,7 +766,7 @@ int64_t IndexerTrack::SaveArtist(db::Connection& dbConnection) {
 void IndexerTrack::SaveDirectory(db::Connection& db, const std::string& filename) {
     try {
         std::string dir = NormalizeDir(
-            boost::filesystem::path(filename).parent_path().string());
+            std::filesystem::path(std::filesystem::u8path(filename)).parent_path().u8string());
 
         int64_t dirId = -1;
         if (metadataIdCache.find("directoryId-" + dir) != metadataIdCache.end()) {
