@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -90,6 +90,11 @@ typedef enum mcsdk_audio_output_code {
     mcsdk_audio_output_error_buffer_written = -1
 } mcsdk_audio_output_code;
 
+typedef enum mcsdk_playback_error_code {
+    mcsdk_playback_error_unknown = -1,
+    mcsdk_playback_error_open_failed = 0,
+} mcsdk_playback_error_code;
+
 typedef enum mcsdk_time_change_mode {
     mcsdk_time_change_mode_seek = 0,
     mcsdk_time_change_mode_scrub = 1
@@ -261,9 +266,9 @@ mcsdk_define_handle(mcsdk_db_transaction);
 typedef struct mcsdk_audio_player_callbacks {
     void (*on_prepared)(mcsdk_audio_player p);
     void (*on_started)(mcsdk_audio_player p);
-    void (*on_almost_ended)(mcsdk_audio_player p);
+    void (*on_stream_eof)(mcsdk_audio_player p);
     void (*on_finished)(mcsdk_audio_player p);
-    void (*on_error)(mcsdk_audio_player p);
+    void (*on_error)(mcsdk_audio_player p, mcsdk_playback_error_code ec);
     void (*on_destroying)(mcsdk_audio_player p);
     void (*on_mixpoint)(mcsdk_audio_player p, int id, double time);
 } mcsdk_audio_player_callbacks;
@@ -450,7 +455,7 @@ mcsdk_export mcsdk_track_list_editor mcsdk_svc_playback_edit_playlist(mcsdk_svc_
 mcsdk_export mcsdk_time_change_mode mcsdk_svc_playback_get_time_change_mode(mcsdk_svc_playback pb);
 mcsdk_export void mcsdk_svc_playback_set_time_change_mode(mcsdk_svc_playback pb, mcsdk_time_change_mode mode);
 mcsdk_export void mcsdk_svc_playback_reload_output(mcsdk_svc_playback pb);
-mcsdk_export mcsdk_track_list mcsdk_svc_playback_clone(mcsdk_svc_playback pb);
+mcsdk_export mcsdk_track_list mcsdk_svc_playback_clone_track_list(mcsdk_svc_playback pb);
 
 /*
  * IPreferences
