@@ -1,6 +1,6 @@
 ﻿//////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -49,7 +49,7 @@
 #include <musikcore/audio/Outputs.h>
 #include <musikcore/support/Messages.h>
 #include <musikcore/sdk/ISchema.h>
-#include <musikcore/version.h>
+#include <musikcore/sdk/version.h>
 
 #include <app/util/Hotkeys.h>
 #include <app/util/Messages.h>
@@ -70,19 +70,16 @@ using namespace musik::core;
 using namespace musik::core::audio;
 using namespace musik::core::library::constants;
 using namespace musik::core::sdk;
+using namespace musik::core::net;
 using namespace musik::cube;
 using namespace cursespp;
 
 #define ENABLE_COLOR_THEME_SELECTION
 
-#ifndef WIN32
+#if !defined(WIN32) || defined(PDCURSES_WINCON)
 #define ENABLE_UNIX_TERMINAL_OPTIONS
 #endif
 
-#ifdef PDCURSES_WINCON
-#undef ENABLE_COLOR_THEME_SELECTION
-#endif
-
 #ifdef WIN32
 #define ENABLE_MINIMIZE_TO_TRAY
 #endif
@@ -108,7 +105,9 @@ static const std::string arrow = "> ";
 
 static inline std::shared_ptr<ISchema> AdvancedSettingsSchema() {
     auto schema = std::make_shared<TSchema<>>();
+#ifndef DISABLE_UPDATE_CHECK
     schema->AddBool(cube::prefs::keys::AutoUpdateCheck, false);
+#endif
 #ifdef ENABLE_MINIMIZE_TO_TRAY
     schema->AddBool(cube::prefs::keys::MinimizeToTray, false);
     schema->AddBool(cube::prefs::keys::StartMinimized, false);
@@ -127,6 +126,8 @@ static inline std::shared_ptr<ISchema> AdvancedSettingsSchema() {
     schema->AddInt(core::prefs::keys::IndexerTransactionInterval, 300);
     schema->AddString(core::prefs::keys::AuddioApiToken, "");
     schema->AddBool(core::prefs::keys::ResumePlaybackOnStartup, false);
+    schema->AddBool(core::prefs::keys::PiggyEnabled, false);
+    schema->AddString(core::prefs::keys::PiggyHostname, "localhost");
     return schema;
 }
 
@@ -297,6 +298,7 @@ void SettingsLayout::OnAdvancedSettingsActivate(cursespp::TextLabel* label) {
 }
 
 void SettingsLayout::OnUpdateDropdownActivate(cursespp::TextLabel* label) {
+#ifndef DISABLE_UPDATE_CHECK
     updateCheck.Run([this](bool updateRequired, std::string version, std::string url) {
         if (updateRequired) {
             UpdateCheck::ShowUpgradeAvailableOverlay(version, url, false);
@@ -305,6 +307,7 @@ void SettingsLayout::OnUpdateDropdownActivate(cursespp::TextLabel* label) {
             UpdateCheck::ShowNoUpgradeFoundOverlay();
         }
     });
+#endif
 }
 
 void SettingsLayout::OnThemeDropdownActivate(cursespp::TextLabel* label) {
@@ -373,8 +376,9 @@ void SettingsLayout::OnLayout() {
     this->saveSessionCheckbox->MoveAndResize(column2, y++, columnCx, kLabelHeight);
     this->pluginsDropdown->MoveAndResize(column2, y++, columnCx, kLabelHeight);
     this->advancedDropdown->MoveAndResize(column2, y++, columnCx, kLabelHeight);
+#ifndef DISABLE_UPDATE_CHECK
     this->updateDropdown->MoveAndResize(column2, y++, columnCx, kLabelHeight);
-
+#endif
     this->appVersion->MoveAndResize(0, cy - 1, cx, kLabelHeight);
 }
 
@@ -432,9 +436,11 @@ void SettingsLayout::InitializeWindows() {
         this->serverDropdown->Activated.connect(this, &SettingsLayout::OnServerDropdownActivate);
     }
 
+#ifndef DISABLE_UPDATE_CHECK
     this->updateDropdown = std::make_shared<TextLabel>();
     this->updateDropdown->SetText(arrow + _TSTR("settings_check_for_updates"));
     this->updateDropdown->Activated.connect(this, &SettingsLayout::OnUpdateDropdownActivate);
+#endif
 
     this->advancedDropdown = std::make_shared<TextLabel>();
     this->advancedDropdown->SetText(arrow + _TSTR("settings_advanced_settings"));
@@ -453,8 +459,6 @@ void SettingsLayout::InitializeWindows() {
     this->appVersion = std::make_shared<TextLabel>();
     this->appVersion->SetContentColor(Color::TextDisabled);
     this->appVersion->SetAlignment(text::AlignCenter);
-    std::string version = u8fmt("%s %s", VERSION, VERSION_COMMIT_HASH);
-    this->appVersion->SetText(u8fmt(_TSTR("console_version"), version.c_str()));
 
     int order = 0;
     this->libraryTypeDropdown->SetFocusOrder(order++);
@@ -485,7 +489,9 @@ void SettingsLayout::InitializeWindows() {
     this->saveSessionCheckbox->SetFocusOrder(order++);
     this->pluginsDropdown->SetFocusOrder(order++);
     this->advancedDropdown->SetFocusOrder(order++);
+#ifndef DISABLE_UPDATE_CHECK
     this->updateDropdown->SetFocusOrder(order++);
+#endif
 
     this->AddWindow(this->libraryTypeDropdown);
     this->AddWindow(this->localLibraryLayout);
@@ -517,7 +523,9 @@ void SettingsLayout::InitializeWindows() {
     this->AddWindow(this->saveSessionCheckbox);
     this->AddWindow(this->pluginsDropdown);
     this->AddWindow(this->advancedDropdown);
+#ifndef DISABLE_UPDATE_CHECK
     this->AddWindow(this->updateDropdown);
+#endif
     this->AddWindow(this->appVersion);
 }
 
@@ -565,13 +573,6 @@ void SettingsLayout::OnRemovedFromParent(IWindow* parent) {
     MessageQueue().UnregisterForBroadcasts(this);
 }
 
-void SettingsLayout::ProcessMessage(musik::core::runtime::IMessage &message) {
-    LayoutBase::ProcessMessage(message);
-    if (message.Type() == core::message::EnvironmentUpdated) {
-        this->LoadPreferences();
-    }
-}
-
 void SettingsLayout::LoadPreferences() {
     this->syncOnStartupCheckbox->SetChecked(this->prefs->GetBool(core::prefs::keys::SyncOnStartup, true));
     this->removeCheckbox->SetChecked(this->prefs->GetBool(core::prefs::keys::RemoveMissingFiles, true));
@@ -641,6 +642,10 @@ void SettingsLayout::LoadPreferences() {
     this->localLibraryLayout->LoadPreferences();
     this->remoteLibraryLayout->LoadPreferences();
 
+    /* version, status */
+    std::string version = u8fmt("%s %s", MUSIKCUBE_VERSION, MUSIKCUBE_VERSION_COMMIT_HASH);
+    this->appVersion->SetText(u8fmt(_TSTR("console_version"), version.c_str()));
+
     this->Layout();
 }
 
