@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -105,6 +105,7 @@ static inline void DrawCursor(IInput* input) {
                 const int targetY = 0;
                 const int targetX = (int) input->Position();
                 wmove(content, targetY, targetX);
+                wnoutrefresh(content);
             }
             return;
         }
@@ -131,8 +132,8 @@ bool Window::WriteToScreen(IInput* input) {
     if (drawPending && !freeze) {
         drawPending = false;
         update_panels();
-        doupdate();
         DrawCursor(input);
+        doupdate();
         return true;
     }
     else if (freeze) {
@@ -327,7 +328,9 @@ void Window::MoveAndResize(int x, int y, int width, int height) {
         this->lastAbsoluteY = absY;
         this->width = width;
         this->height = height;
-        this->RecreateForUpdatedDimensions();
+        if (this->isVisibleInParent) {
+            this->RecreateForUpdatedDimensions();
+        }
     }
     else {
         this->DestroyIfBadBounds();
@@ -616,7 +619,7 @@ void Window::Show() {
             }
         }
         else {
-            this->Create();
+            this->RecreateForUpdatedDimensions();
             this->isVisibleInParent = true;
         }
 
@@ -965,7 +968,8 @@ void Window::SetNavigationKeys(std::shared_ptr<INavigationKeys> keys) {
     ::keys = keys;
 }
 
-bool Window::MouseEvent(const IMouseHandler::Event& mouseEvent) {
+bool Window::ProcessMouseEvent(const IMouseHandler::Event& mouseEvent) {
+    this->MouseEvent(this, &mouseEvent);
     return false;
 }
 
