@@ -1,6 +1,5 @@
 package io.casey.musikcube.remote.framework
 
-import android.content.Intent
 import android.os.Bundle
 
 interface IMixin {
@@ -10,6 +9,5 @@ interface IMixin {
     fun onPause()
     fun onStop()
     fun onSaveInstanceState(bundle: Bundle)
-    fun onActivityResult(request: Int, result: Int, data: Intent?)
     fun onDestroy()
 }
\ No newline at end of file
