@@ -1,13 +1,12 @@
 package io.casey.musikcube.remote.service.system
 
+import android.annotation.SuppressLint
 import android.app.*
 import android.content.*
 import android.graphics.Bitmap
 import android.graphics.drawable.Drawable
 import android.media.AudioManager
-import android.os.Build
-import android.os.IBinder
-import android.os.PowerManager
+import android.os.*
 import android.support.v4.media.MediaMetadataCompat
 import android.support.v4.media.session.MediaSessionCompat
 import android.support.v4.media.session.PlaybackStateCompat
@@ -33,10 +32,19 @@ import io.casey.musikcube.remote.service.websocket.model.ITrack
 import io.casey.musikcube.remote.ui.home.activity.MainActivity
 import io.casey.musikcube.remote.ui.settings.constants.Prefs
 import io.casey.musikcube.remote.ui.shared.extension.fallback
+import io.casey.musikcube.remote.ui.shared.extension.getParcelableExtraCompat
+import io.casey.musikcube.remote.ui.shared.util.AlbumArtLookup
 import io.casey.musikcube.remote.ui.shared.util.Size
 import io.casey.musikcube.remote.util.Debouncer
 import androidx.core.app.NotificationCompat.Action as NotifAction
-import io.casey.musikcube.remote.ui.shared.util.AlbumArtLookup.getUrl as getAlbumArtUrl
+
+const val ENABLE_LOGGING = false
+
+private fun log(format: String, vararg params: Any?) {
+    if (ENABLE_LOGGING) {
+        Log.d("musikdroid.Service", String.format(format, *params))
+    }
+}
 
 /**
  * a service used to interact with all of the system media-related components -- notifications,
@@ -61,11 +69,12 @@ class SystemService : Service() {
     private lateinit var powerManager: PowerManager
     private lateinit var prefs: SharedPreferences
 
+    private val handler = Handler(Looper.getMainLooper())
     private val albumArt = AlbumArt()
     private val sessionData = SessionMetadata()
 
     override fun onCreate() {
-        Log.d(TAG, "onCreate")
+        log("onCreate")
 
         state = State.Sleeping
 
@@ -93,7 +102,7 @@ class SystemService : Service() {
     }
 
     override fun onDestroy() {
-        Log.d(TAG, "onDestroy")
+        log("onDestroy")
 
         state = State.Dead
         super.onDestroy()
@@ -105,7 +114,7 @@ class SystemService : Service() {
     }
 
     override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
-        Log.d(TAG, "onStartCommand")
+        log("onStartCommand")
 
         if (intent != null && intent.action != null) {
             when (intent.action) {
@@ -126,7 +135,7 @@ class SystemService : Service() {
     override fun onBind(intent: Intent): IBinder? = null
 
     private fun wakeupNow() {
-        Log.d(TAG, "SystemService WAKE_UP")
+        log("SystemService WAKE_UP")
 
         val sleeping = playback == null || wakeLock == null
 
@@ -141,13 +150,13 @@ class SystemService : Service() {
         }
 
         checkInitMediaSession()
-        updateMediaSessionPlaybackState()
+        updateNotificationAndSessionDebouncer.call()
 
         state = State.Active
     }
 
     private fun shutdownNow() {
-        Log.d(TAG, "SystemService SHUT_DOWN")
+        log("SystemService SHUT_DOWN")
 
         deinitMediaSession()
 
@@ -160,12 +169,13 @@ class SystemService : Service() {
     }
 
     private fun sleepNow() {
-        Log.d(TAG, "SystemService SLEEP")
+        log("SystemService SLEEP")
         releaseWakeLock()
         playback?.disconnect(playbackListener)
         state = State.Sleeping
     }
 
+    @SuppressLint("WakelockTimeout")
     private fun acquireWakeLock() {
         if (wakeLock == null) {
             wakeLock = powerManager.newWakeLock(
@@ -189,12 +199,14 @@ class SystemService : Service() {
     }
 
     private fun checkInitMediaSession() {
+        log("checkInitMediaSession: isActive=${mediaSession?.isActive}")
         if (mediaSession == null || mediaSession?.isActive != true) {
             deinitMediaSession()
 
             val receiver = ComponentName(
                 packageName, MediaButtonReceiver::class.java.name)
 
+            log("checkInitMediaSession: creating new session")
             mediaSession = MediaSessionCompat(
                 this, SESSION_TAG, receiver, null)
                 .apply {
@@ -205,6 +217,7 @@ class SystemService : Service() {
     }
 
     private fun deinitMediaSession() {
+        log("deinitMediaSession: destroying session")
         mediaSession?.release()
         mediaSession = null
     }
@@ -219,46 +232,17 @@ class SystemService : Service() {
             unregisterReceiver(headsetUnpluggedReceiver)
         }
         catch (ex: Exception) {
-            Log.e(TAG, "unable to unregister headset (un)plugged BroadcastReceiver")
+            log("unregisterReceivers: unable to unregister headset (un)plugged BroadcastReceiver")
         }
 
-    private fun updateMediaSessionPlaybackState() {
-        mediaSession?.let { session ->
-            var mediaSessionState = PlaybackStateCompat.STATE_STOPPED
-            var duration = 0
-            var playing: ITrack? = null
-
-            playback?.let { pb ->
-                mediaSessionState = when (pb.state) {
-                    PlaybackState.Playing -> PlaybackStateCompat.STATE_PLAYING
-                    PlaybackState.Buffering -> PlaybackStateCompat.STATE_BUFFERING
-                    /* HACK: if we return STATE_PAUSED, the playback controls will disappear
-                    from the lock screen! wtf? we don't want that, so we return
-                    STATE_BUFFERING instead, which seems to keep the play controls visible,
-                    and also display the "PLAY" button like the user expects. sigh */
-                    PlaybackState.Paused -> PlaybackStateCompat.STATE_BUFFERING
-                    PlaybackState.Stopped -> PlaybackStateCompat.STATE_STOPPED
-                }
-                playing = pb.playingTrack
-                duration = (pb.duration * 1000).toInt()
-            }
-
-            updateMediaSession(playing, duration)
-            updateNotification(playing, playback?.state ?: PlaybackState.Stopped)
-
-            session.setPlaybackState(PlaybackStateCompat.Builder()
-                .setState(mediaSessionState, 0, 0f)
-                .setActions(MEDIA_SESSION_ACTIONS)
-                .build())
-        }
-    }
-
     private fun downloadAlbumArtIfNecessary(track: ITrack, duration: Int) {
         if (!albumArt.same(track) || (albumArt.request == null && albumArt.bitmap == null)) {
-            if (track.artist.isNotBlank() && track.album.isNotBlank()) {
-                Log.d(TAG, "downloading album art")
+            log("downloadAlbumArtIfNecessary: detected different different track")
 
-                val url = getAlbumArtUrl(track, Size.Mega)
+            if (track.artist.isNotBlank() && track.album.isNotBlank()) {
+                log("downloadAlbumArtIfNecessary: metadata available, attempting to download artwork")
+
+                val url = AlbumArtLookup.getUrl(track, Size.Mega)
 
                 val request = GlideApp.with(applicationContext)
                     .asBitmap()
@@ -273,13 +257,18 @@ class SystemService : Service() {
                             /* make sure the instance's current request is the same as this request. it's
                             possible we had another download request come in before this one finished */
                             if (albumArt.request == request) {
+                                log("downloadAlbumArtIfNecessary: artwork downloaded")
                                 albumArt.bitmap = bitmap
                                 albumArt.request = null
                                 updateMediaSession(track, duration)
                             }
+                            else {
+                                log("downloadAlbumArtIfNecessary: artwork downloaded, but request does not match!")
+                            }
                         }
 
                         override fun onLoadFailed(errorDrawable: Drawable?) {
+                            log("downloadAlbumArtIfNecessary: artwork failed to download. albumArt.request=${albumArt.request}, request=$request")
                             if (albumArt.request == request) {
                                 albumArt.request = null
                             }
@@ -288,7 +277,7 @@ class SystemService : Service() {
             }
         }
         else {
-            Log.d(TAG, "downloadAlbumArt already in flight")
+            log("downloadAlbumArtIfNecessary: downloadAlbumArt already in flight")
         }
     }
 
@@ -300,19 +289,11 @@ class SystemService : Service() {
             currentImage = albumArt.bitmap
         }
 
-        if (!sessionData.matches(track, currentImage)) {
+        if (!sessionData.matches(track, currentImage, duration)) {
+            log("updateMediaSession: stale data detected, updating")
             sessionData.update(track, currentImage, duration)
-            mediaSessionDebouncer.call()
-        }
-    }
 
-    /* some versions of the OS seem to have problems if we update the lockscreen
-    artwork more quickly than twice a second. this can happen if we just loaded
-    a new track but don't have artwork yet, but artwork comes back right away.
-    this debouncer ensures we wait at least half a second between updates */
-    private val mediaSessionDebouncer = object: Debouncer<Unit>(500) {
-        override fun onDebounced(last: Unit?) {
-            val track = sessionData.track
+            log("updateMediaSession: updating with title=${track?.title}, album=${track?.album}, artist=${track?.artist}")
             mediaSession?.setMetadata(MediaMetadataCompat.Builder()
                 .putString(MediaMetadataCompat.METADATA_KEY_ARTIST, track?.artist ?: "-")
                 .putString(MediaMetadataCompat.METADATA_KEY_ALBUM, track?.album ?: "-")
@@ -321,16 +302,97 @@ class SystemService : Service() {
                 .putBitmap(MediaMetadataCompat.METADATA_KEY_ALBUM_ART, sessionData.bitmap)
                 .build())
         }
+        else {
+            log("updateMediaSession: data up to date, not updating")
+        }
     }
 
+    /* Updating either MediaSession or the NotificationCompat.MediaStyle too quickly (or from different
+    threads) can spin it into a weird, unsynchronized state. Let's ensure we debounce updates to
+    these system-provided APIs. */
+    private val updateNotificationAndSessionDebouncer = object: Debouncer<Unit>(NOTIFICATION_DEBOUNCE_MS) {
+        override fun onDebounced(last: Unit?) {
+            mediaSession?.let { session ->
+                var mediaSessionState = PlaybackStateCompat.STATE_STOPPED
+                var duration = 0
+                var position = 0L
+                var bufferPosition = 0L
+                var playing: ITrack? = null
+
+                playback?.let { pb ->
+                    mediaSessionState = when (pb.state) {
+                        PlaybackState.Playing -> PlaybackStateCompat.STATE_PLAYING
+                        PlaybackState.Buffering -> PlaybackStateCompat.STATE_BUFFERING
+                        PlaybackState.Paused -> PlaybackStateCompat.STATE_PAUSED
+                        PlaybackState.Stopped -> PlaybackStateCompat.STATE_STOPPED
+                    }
+                    playing = pb.playingTrack
+                    duration = (pb.duration * 1000).toInt()
+                    position = (pb.currentTime * 1000).toLong()
+                    bufferPosition = (pb.bufferedTime * 1000).toLong()
+                }
+
+                updateMediaSession(playing, duration)
+                updateNotification(playing, playback?.state ?: PlaybackState.Stopped)
+
+                session.setPlaybackState(PlaybackStateCompat.Builder()
+                    .setState(mediaSessionState, position, 1f)
+                    .setBufferedPosition(bufferPosition)
+                    .setActions(MEDIA_SESSION_ACTIONS)
+                    .build())
+            }
+        }
+    }
+
+    private fun scheduleNotificationTimeUpdate() {
+        handler.removeCallbacks(updateTimeRunnable)
+        handler.postDelayed(updateTimeRunnable, NOTIFICATION_PLAYHEAD_SYNC_MS)
+    }
+
+    private val updateTimeRunnable = object: Runnable {
+        override fun run() {
+            playback?.let { pb ->
+                when (pb.state) {
+                    PlaybackState.Playing -> {
+                        updateNotificationAndSessionDebouncer.call()
+                        scheduleNotificationTimeUpdate()
+                    }
+                    PlaybackState.Buffering,
+                    PlaybackState.Paused,
+                    PlaybackState.Stopped -> {
+                    }
+                }
+            }
+        }
+    }
+
+    @SuppressLint("UnspecifiedImmutableFlag")
+    private fun getPendingActivityIntentCompat(intent: Intent) =
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
+            PendingIntent.getActivity(applicationContext, 1, intent, PendingIntent.FLAG_IMMUTABLE)
+        }
+        else {
+            PendingIntent.getActivity(applicationContext, 1, intent, 0)
+        }
+
+    @SuppressLint("UnspecifiedImmutableFlag")
+    private fun getPendingServiceIntentCompat(intent: Intent) =
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
+            PendingIntent.getService(applicationContext, 1, intent, PendingIntent.FLAG_IMMUTABLE)
+        }
+        else {
+            PendingIntent.getService(applicationContext, 1, intent, 0)
+        }
+
     private fun updateNotification(track: ITrack?, state: PlaybackState) {
-        val contentIntent = PendingIntent.getActivity(
-            applicationContext, 1, MainActivity.getStartIntent(this), 0)
+        val contentIntent = getPendingActivityIntentCompat(MainActivity.getStartIntent(this))
 
         val title = fallback(track?.title, "-")
         val artist = fallback(track?.artist, "-")
         val album = fallback(track?.album, "-")
 
+        log("updateNotification: state=$state title=$title, album=$album, artist=$artist")
+
         val notification = NotificationCompat.Builder(this, NOTIFICATION_CHANNEL)
             .setSmallIcon(R.drawable.ic_notification)
             .setContentTitle(title)
@@ -382,14 +444,13 @@ class SystemService : Service() {
             }
         }
 
+        notificationManager.cancel(NOTIFICATION_ID)
         startForeground(NOTIFICATION_ID, notification.build())
     }
 
     private fun action(icon: Int, title: String, intentAction: String): NotifAction {
-        val intent = Intent(applicationContext, SystemService::class.java)
-        intent.action = intentAction
-        val pendingIntent = PendingIntent.getService(applicationContext, 1, intent, 0)
-        return NotifAction.Builder(icon, title, pendingIntent).build()
+        val intent = Intent(applicationContext, SystemService::class.java).apply { action = intentAction }
+        return NotifAction.Builder(icon, title, getPendingServiceIntentCompat(intent)).build()
     }
 
     private fun handlePlaybackAction(action: String?): Boolean {
@@ -451,7 +512,9 @@ class SystemService : Service() {
     private val mediaSessionCallback = object : MediaSessionCompat.Callback() {
         override fun onMediaButtonEvent(mediaButtonEvent: Intent?): Boolean {
             if (Intent.ACTION_MEDIA_BUTTON == mediaButtonEvent?.action) {
-                val event = mediaButtonEvent.getParcelableExtra<KeyEvent>(Intent.EXTRA_KEY_EVENT) ?: return super.onMediaButtonEvent(mediaButtonEvent)
+                val event =
+                    mediaButtonEvent.getParcelableExtraCompat<KeyEvent>(Intent.EXTRA_KEY_EVENT) ?:
+                    return super.onMediaButtonEvent(mediaButtonEvent)
 
                 val keycode = event.keyCode
                 val action = event.action
@@ -501,6 +564,10 @@ class SystemService : Service() {
             return false
         }
 
+        override fun onSeekTo(pos: Long) {
+            playback?.seekTo(pos.toDouble() / 1000.0)
+        }
+
         override fun onPlay() {
             playback?.let {
                 when (it.queueCount == 0) {
@@ -536,7 +603,8 @@ class SystemService : Service() {
         if (playback?.state == PlaybackState.Playing) {
             headsetDoublePauseHack = false
         }
-        updateMediaSessionPlaybackState()
+        scheduleNotificationTimeUpdate()
+        updateNotificationAndSessionDebouncer.call()
     }
 
     private val headsetUnpluggedReceiver = object : BroadcastReceiver() {
@@ -568,6 +636,7 @@ class SystemService : Service() {
         var track: ITrack? = null
 
         fun reset(t: ITrack? = null) {
+            log("AlbumArt.reset()")
             bitmap = null
             request = null
             target = null
@@ -589,20 +658,33 @@ class SystemService : Service() {
             duration = otherDuration
         }
 
-        fun matches(otherTrack: ITrack?, otherBitmap: Bitmap?): Boolean {
-            return (track != null && otherTrack != null) &&
+        fun matches(otherTrack: ITrack?, otherBitmap: Bitmap?, otherDuration: Int): Boolean {
+            log("updateMediaSession.matches(): " +
+                    "track=$track, " +
+                    "otherTrack=$otherTrack " +
+                    "otherTrack.externalId=${otherTrack?.externalId} " +
+                    "track.externalId=${track?.externalId} " +
+                    "otherBitmap=$otherBitmap " +
+                    "bitmap=$bitmap" +
+                    "otherDuration=$otherDuration " +
+                    "duration=$duration")
+            val result = (track != null && otherTrack != null) &&
                 otherTrack.externalId == track?.externalId &&
-                bitmap === otherBitmap
+                bitmap === otherBitmap &&
+                duration == otherDuration
+            log("updateMediaSession.matches(): result=$result")
+            return result
         }
     }
 
     companion object {
-        private const val TAG = "SystemService"
         private const val SESSION_TAG = "musikdroid.SystemService"
         private const val NOTIFICATION_ID = 0xdeadbeef.toInt()
         private const val NOTIFICATION_CHANNEL = "musikdroid"
         private const val HEADSET_HOOK_DEBOUNCE_MS = 500L
         private const val HEADSET_DOUBLE_PAUSE_HACK_DEBOUNCE_MS = 3500L
+        private const val NOTIFICATION_DEBOUNCE_MS = 750L
+        private const val NOTIFICATION_PLAYHEAD_SYNC_MS = 10000L
         private const val ACTION_NOTIFICATION_PLAY = "io.casey.musikcube.remote.NOTIFICATION_PLAY"
         private const val ACTION_NOTIFICATION_PAUSE = "io.casey.musikcube.remote.NOTIFICATION_PAUSE"
         private const val ACTION_NOTIFICATION_NEXT = "io.casey.musikcube.remote.NOTIFICATION_NEXT"
@@ -633,8 +715,8 @@ class SystemService : Service() {
             PlaybackStateCompat.ACTION_PLAY_PAUSE or
             PlaybackStateCompat.ACTION_SKIP_TO_NEXT or
             PlaybackStateCompat.ACTION_SKIP_TO_PREVIOUS or
-            PlaybackStateCompat.ACTION_FAST_FORWARD or
-            PlaybackStateCompat.ACTION_REWIND
+            PlaybackStateCompat.ACTION_STOP or
+            PlaybackStateCompat.ACTION_SEEK_TO
 
         private var wakeLockAcquireTime = -1L
         private var totalWakeLockTime = 0L
