@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -36,6 +36,7 @@
 #include <musikcore/sdk/IDebug.h>
 #include <algorithm>
 #include <string>
+#include <unordered_set>
 
 #ifdef WIN32
 #define DLLEXPORT __declspec(dllexport)
@@ -52,6 +53,10 @@ using namespace musik::core::sdk;
 static const char* TAG = "ffmpegdecoder";
 static IDebug* debug = nullptr;
 
+static std::unordered_set<AVCodecID> ignoreInvalidPacketCodecs = { AV_CODEC_ID_APE };
+
+#define RESOLVE_SAMPLE_RATE() this->preferredSampleRate > 0 ? this->preferredSampleRate : this->rate
+
 extern "C" DLLEXPORT void SetDebug(IDebug* debug) {
     ::debug = debug;
 }
@@ -146,17 +151,13 @@ FfmpegDecoder::FfmpegDecoder() {
     this->decodedFrame = nullptr;
     this->resampledFrame = nullptr;
     this->resampler = nullptr;
-    this->bufferSize = AV_INPUT_BUFFER_PADDING_SIZE + BUFFER_SIZE;
-    this->buffer = new unsigned char[this->bufferSize];
     this->outputFifo = nullptr;
+    this->preferredSampleRate = -1;
 }
 
 FfmpegDecoder::~FfmpegDecoder() {
     this->Reset();
 
-    delete[] this->buffer;
-    this->buffer = nullptr;
-
     if (this->decodedFrame) {
         av_frame_free(&this->decodedFrame);
         this->decodedFrame = nullptr;
@@ -192,12 +193,12 @@ double FfmpegDecoder::SetPosition(double seconds) {
 
 bool FfmpegDecoder::GetBuffer(IBuffer *buffer) {
     if (this->ioContext) {
-        buffer->SetSampleRate((long) this->rate);
+        buffer->SetSampleRate((long) RESOLVE_SAMPLE_RATE());
         buffer->SetChannels((long) this->channels);
         buffer->SetSamples(0);
 
         if (!this->eof) {
-            if (!this->resampler && !this->InitializeResampler(buffer)) {
+            if (!this->resampler && !this->InitializeResampler()) {
                 this->exhausted = true;
                 logError("unable to initialize resampler. marking as done.");
                 return false;
@@ -227,6 +228,7 @@ double FfmpegDecoder::GetDuration() {
 
 void FfmpegDecoder::Reset() {
     if (this->ioContext) {
+        av_free(this->ioContext->buffer);
         av_free(this->ioContext);
         this->ioContext = nullptr;
     }
@@ -248,15 +250,26 @@ void FfmpegDecoder::Reset() {
         av_audio_fifo_free(this->outputFifo);
         this->outputFifo = nullptr;
     }
+    if (this->resampler) {
+        swr_free(&this->resampler);
+        this->resampler = nullptr;
+    }
     this->streamId = -1;
 }
 
-bool FfmpegDecoder::InitializeResampler(IBuffer* buffer) {
+bool FfmpegDecoder::InitializeResampler() {
+    if (this->resampler) {
+        swr_free(&this->resampler);
+        this->resampler = nullptr;
+    }
+
+    int outSampleRate = (int) RESOLVE_SAMPLE_RATE();
+
     this->resampler = swr_alloc_set_opts(
         this->resampler,
         this->codecContext->channel_layout,
         AV_SAMPLE_FMT_FLT,
-        (int) this->rate,
+        outSampleRate,
         this->codecContext->channel_layout,
         this->codecContext->sample_fmt,
         this->codecContext->sample_rate,
@@ -278,9 +291,12 @@ bool FfmpegDecoder::Open(musik::core::sdk::IDataStream *stream) {
 
         this->stream = stream;
 
+        const int ioContextBufferSize = AV_INPUT_BUFFER_PADDING_SIZE + BUFFER_SIZE;
+        unsigned char* ioContextBuffer = (unsigned char*) av_malloc(ioContextBufferSize);
+
         this->ioContext = avio_alloc_context(
-            this->buffer,
-            (int) this->bufferSize,
+            ioContextBuffer,
+            ioContextBufferSize,
             0,
             this,
             readCallback,
@@ -307,7 +323,7 @@ bool FfmpegDecoder::Open(musik::core::sdk::IDataStream *stream) {
 
             if (this->formatContext->iformat) {
                 if (avformat_open_input(&this->formatContext, "", nullptr, nullptr) == 0) {
-                    AVCodec* codec = nullptr;
+                    AVCodecCompat* codec = nullptr;
                     if (avformat_find_stream_info(this->formatContext, nullptr) >= 0) {
                         this->streamId = av_find_best_stream(
                             this->formatContext,
@@ -352,17 +368,20 @@ bool FfmpegDecoder::Open(musik::core::sdk::IDataStream *stream) {
                                 this->codecContext->channel_layout =
                                     av_get_default_channel_layout(this->codecContext->channels);
                             }
+
+                            this->preferredFrameSize = this->codecContext->frame_size
+                                ? this->codecContext->frame_size
+                                : DEFAULT_FRAME_SIZE;
+
+                            this->disableInvalidPacketDetection =
+                                ignoreInvalidPacketCodecs.find(this->codecContext->codec_id) !=
+                                ignoreInvalidPacketCodecs.end();
                         }
 
                         auto stream = this->formatContext->streams[this->streamId];
                         this->rate = stream->codecpar->sample_rate;
                         this->channels = stream->codecpar->channels;
                         this->duration = (double) this->formatContext->duration / (double) AV_TIME_BASE;
-
-                        this->preferredFrameSize = this->codecContext->frame_size
-                            ? this->codecContext->frame_size
-                            : DEFAULT_FRAME_SIZE;
-
                         this->outputFifo = av_audio_fifo_alloc(AV_SAMPLE_FMT_FLT, channels, 1);
 
                         if (!this->outputFifo) {
@@ -404,7 +423,7 @@ bool FfmpegDecoder::ReadSendAndReceivePacket(AVPacket* packet) {
             this->resampledFrame = this->AllocFrame(
                 this->resampledFrame,
                 AV_SAMPLE_FMT_FLT,
-                this->rate,
+                RESOLVE_SAMPLE_RATE(),
                 this->decodedFrame->nb_samples);
 
             error = swr_convert_frame(
@@ -414,8 +433,14 @@ bool FfmpegDecoder::ReadSendAndReceivePacket(AVPacket* packet) {
 
             if (error < 0) {
                 logAvError("swr_convert_frame", error);
+                this->InitializeResampler();
+                error = swr_convert_frame(
+                    this->resampler,
+                    this->resampledFrame,
+                    this->decodedFrame);
             }
-            else {
+
+            if (error >= 0) {
                 error = av_audio_fifo_write(
                     this->outputFifo,
                     (void**) this->resampledFrame->extended_data,
@@ -426,6 +451,10 @@ bool FfmpegDecoder::ReadSendAndReceivePacket(AVPacket* packet) {
                     return false;
                 }
 
+                /* there may still be pending data in the resampler, go ahead
+                and write to the fifo right now so it doesn't back up */
+                this->DrainResamplerToFifoQueue();
+
                 decodedAtLeastOneFrame = true;
             }
         }
@@ -438,8 +467,12 @@ bool FfmpegDecoder::DrainResamplerToFifoQueue() {
         return false;
     }
 
-    int64_t bufferedFrames = swr_get_delay(
-        this->resampler, this->codecContext->sample_rate);
+    const int64_t targetRate =
+        RESOLVE_SAMPLE_RATE() > this->codecContext->sample_rate
+            ? RESOLVE_SAMPLE_RATE()
+            : this->codecContext->sample_rate;
+
+    int64_t bufferedFrames = swr_get_delay(this->resampler, targetRate);
 
     while (bufferedFrames > 0) {
         this->resampledFrame = this->AllocFrame(
@@ -472,9 +505,6 @@ bool FfmpegDecoder::DrainResamplerToFifoQueue() {
         }
     }
 
-    swr_free(&this->resampler);
-    this->resampler = nullptr;
-
     return true;
 }
 
@@ -494,8 +524,15 @@ bool FfmpegDecoder::RefillFifoQueue() {
             /* note that sometimes decoders seem to return packets that are
             invalid. this can be observed when playing wav files that have
             album art metadata, but may happen in other cases. if we detect
-            an invalid packet, simply discard it and get the next one */
-            if (packet.pos == -1 && packet.duration <= 1) {
+            an invalid packet, simply discard it and get the next one. to make
+            things worse, some decoders (e.g. APE) always return packets in this
+            state. we're missing something, somewhere during codec init probably,
+            but after hours of messing around I can't figure it out. if you
+            are reading this comment and have any ideas, please let me know. */
+            if (packet.pos == -1 &&
+                packet.duration <= 1 &&
+                !this->disableInvalidPacketDetection)
+            {
                 logError("invalid packet detected, discarding.");
             }
             else {
@@ -554,8 +591,11 @@ AVFrame* FfmpegDecoder::AllocFrame(AVFrame* original, AVSampleFormat format, int
         if (original || frameSizeChanged) {
             av_frame_free(&original);
         }
+        const int channelLayout = this->codecContext->channel_layout == 0
+            ? av_get_default_channel_layout(this->codecContext->channels)
+            : this->codecContext->channel_layout;
         original = av_frame_alloc();
-        original->channel_layout = this->codecContext->channel_layout;
+        original->channel_layout = channelLayout;
         original->format = format;
         original->sample_rate = sampleRate;
         if (frameSizeChanged) {
@@ -564,4 +604,4 @@ AVFrame* FfmpegDecoder::AllocFrame(AVFrame* original, AVSampleFormat format, int
         }
     }
     return original;
-}
\ No newline at end of file
+}
