@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -34,13 +34,15 @@
 
 #include "FfmpegEncoder.h"
 
-#ifdef ENABLE_FFMPEG
-
 #include "shared.h"
 #include <algorithm>
 #include <random>
 #include <map>
 
+#if LIBAVUTIL_VERSION_MAJOR >= 58
+#define USE_FFMPEG6_CHANNEL_LAYOUT
+#endif
+
 using namespace musik::core::sdk;
 
 static const int IO_CONTEXT_BUFFER_SIZE = 4096;
@@ -89,7 +91,7 @@ static void logError(const std::string& message) {
     }
 }
 
-static AVSampleFormat resolveSampleFormat(AVCodec *codec) {
+static AVSampleFormat resolveSampleFormat(AVCodecCompat *codec) {
     if (!codec->sample_fmts) {
         return AV_SAMPLE_FMT_NONE;
     }
@@ -105,7 +107,7 @@ static AVSampleFormat resolveSampleFormat(AVCodec *codec) {
     return codec->sample_fmts[0];
 }
 
-static int resolveSampleRate(AVCodec* codec, int preferredSampleRate) {
+static int resolveSampleRate(AVCodecCompat* codec, int preferredSampleRate) {
     const int *p;
     int highestRate = 0;
     if (!codec->supported_samplerates) {
@@ -274,8 +276,13 @@ bool FfmpegEncoder::OpenOutputCodec(size_t rate, size_t channels, size_t bitrate
         return false;
     }
 
-    this->outputContext->channels = (int) channels;
+#ifdef USE_FFMPEG6_CHANNEL_LAYOUT
+    this->outputContext->ch_layout.nb_channels = (int)channels;
+    this->outputContext->ch_layout.order = AV_CHANNEL_ORDER_NATIVE;
+#else
+    this->outputContext->channels = (int)channels;
     this->outputContext->channel_layout = resolveChannelLayout(channels);
+#endif
     this->outputContext->sample_rate = resolveSampleRate(this->outputCodec, (int) rate);
     this->outputContext->sample_fmt = resolveSampleFormat(this->outputCodec);
     this->outputContext->bit_rate = (int64_t) bitrate * 1000;
@@ -316,6 +323,18 @@ bool FfmpegEncoder::OpenOutputCodec(size_t rate, size_t channels, size_t bitrate
 
     /* resampler context that will be used to convert the input audio
     sample format to the one recommended by the encoder */
+#ifdef USE_FFMPEG6_CHANNEL_LAYOUT
+    swr_alloc_set_opts2(
+        &this->resampler,
+        &this->outputContext->ch_layout,
+        this->outputContext->sample_fmt,
+        this->outputContext->sample_rate,
+        &this->outputContext->ch_layout,
+        AV_SAMPLE_FMT_FLT,
+        (int)rate,
+        0,
+        nullptr);
+#else
     this->resampler = swr_alloc_set_opts(
         nullptr,
         this->outputContext->channel_layout,
@@ -326,6 +345,8 @@ bool FfmpegEncoder::OpenOutputCodec(size_t rate, size_t channels, size_t bitrate
         (int)rate,
         0,
         nullptr);
+#endif
+
 
     error = swr_init(this->resampler);
 
@@ -361,10 +382,6 @@ bool FfmpegEncoder::OpenOutputContext() {
 }
 
 bool FfmpegEncoder::Initialize(IDataStream* out, size_t rate, size_t channels, size_t bitrate) {
-#ifndef WIN32
-    av_register_all();
-#endif
-
     this->out = out;
 
     if (this->OpenOutputContext()) {
@@ -626,9 +643,13 @@ AVFrame* FfmpegEncoder::ReallocFrame(
         }
         original = av_frame_alloc();
         original->nb_samples = samplesPerChannel;
-        original->channel_layout = this->outputContext->channel_layout;
         original->format = format;
         original->sample_rate = sampleRate;
+#ifdef USE_FFMPEG6_CHANNEL_LAYOUT
+        original->ch_layout = this->outputContext->ch_layout;
+#else
+        original->channel_layout = this->outputContext->channel_layout;
+#endif
         int error = av_frame_get_buffer(original, 0);
         if (error < 0) {
             logAvError("av_frame_get_buffer", error);
@@ -637,5 +658,3 @@ AVFrame* FfmpegEncoder::ReallocFrame(
     }
     return original;
 }
-
-#endif // ENABLE_FFMPEG
\ No newline at end of file
