@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -34,117 +34,65 @@
 
 #include "TaglibMetadataReader.h"
 
-#ifdef WIN32
-    #include <taglib/toolkit/tlist.h>
-    #include <taglib/toolkit/tfile.h>
-    #include <taglib/tag.h>
-    #include <taglib/fileref.h>
-    #include <taglib/audioproperties.h>
-    #include <taglib/mpeg/mpegfile.h>
-    #include <taglib/mpeg/id3v1/id3v1tag.h>
-    #include <taglib/mpeg/id3v1/id3v1genres.h>
-    #include <taglib/mpeg/id3v2/id3v2tag.h>
-    #include <taglib/mpeg/id3v2/id3v2header.h>
-    #include <taglib/mpeg/id3v2/id3v2frame.h>
-    #include <taglib/mpeg/id3v2/frames/attachedpictureframe.h>
-    #include <taglib/mpeg/id3v2/frames/commentsframe.h>
-    #include <taglib/mpeg/id3v2/frames/textidentificationframe.h>
-    #include <taglib/mp4/mp4file.h>
-    #include <taglib/ogg/oggfile.h>
-    #include <taglib/ogg/xiphcomment.h>
-    #include <taglib/ogg/opus/opusfile.h>
-    #include <taglib/flac/flacfile.h>
-    #include <taglib/toolkit/tpropertymap.h>
-    #include <taglib/wavpack/wavpackfile.h>
-    #include <taglib/riff/aiff/aifffile.h>
-    #include <taglib/riff/wav/wavfile.h>
-#else
-    #include <taglib/tlist.h>
-    #include <taglib/tfile.h>
-    #include <taglib/tag.h>
-    #include <taglib/fileref.h>
-    #include <taglib/audioproperties.h>
-    #include <taglib/mpegfile.h>
-    #include <taglib/id3v1tag.h>
-    #include <taglib/id3v1genres.h>
-    #include <taglib/id3v2tag.h>
-    #include <taglib/id3v2header.h>
-    #include <taglib/id3v2frame.h>
-    #include <taglib/attachedpictureframe.h>
-    #include <taglib/commentsframe.h>
-    #include <taglib/mp4file.h>
-    #include <taglib/oggfile.h>
-    #include <taglib/opusfile.h>
-    #include <taglib/flacfile.h>
-    #include <taglib/wavpackfile.h>
-    #include <taglib/xiphcomment.h>
-    #include <taglib/tpropertymap.h>
-    #include <taglib/aifffile.h>
-    #include <taglib/wavfile.h>
-    #include <taglib/textidentificationframe.h>
-#endif
+#include <musikcore/sdk/String.h>
+
+#include <taglib/tlist.h>
+#include <taglib/tfile.h>
+#include <taglib/tag.h>
+#include <taglib/fileref.h>
+#include <taglib/audioproperties.h>
+#include <taglib/mpegfile.h>
+#include <taglib/id3v1tag.h>
+#include <taglib/id3v1genres.h>
+#include <taglib/id3v2tag.h>
+#include <taglib/id3v2header.h>
+#include <taglib/id3v2frame.h>
+#include <taglib/attachedpictureframe.h>
+#include <taglib/commentsframe.h>
+#include <taglib/mp4file.h>
+#include <taglib/oggfile.h>
+#include <taglib/opusfile.h>
+#include <taglib/flacfile.h>
+#include <taglib/wavpackfile.h>
+#include <taglib/xiphcomment.h>
+#include <taglib/tpropertymap.h>
+#include <taglib/aifffile.h>
+#include <taglib/wavfile.h>
+#include <taglib/textidentificationframe.h>
 
 #include <vector>
 #include <string>
+#include <set>
 #include <iostream>
 #include <functional>
 #include <cctype>
 #include <algorithm>
 #include <string.h>
 
-#ifdef WIN32
-#define ENABLE_FFMPEG
-#endif
-
 using namespace musik::core::sdk;
 
-namespace str {
-    static std::string lower(std::string input) {
-        std::transform(input.begin(), input.end(), input.begin(), ::tolower);
-        return input;
-    }
+static std::set<std::string> SUPPORTED_FORMATS = {
+    "aac",
+    "aif",
+    "aiff",
+    "alac",
+    "ape",
+    "flac",
+    "m4a",
+    "mp3",
+    "mpc",
+    "ogg",
+    "opus",
+    "wav",
+    "wave",
+    "wma",
+    "wv",
+};
 
-    static inline bool isSpace(const char c) {
-        return c == ' ' || c == '\n' || c == '\r' || c == '\t' || c == '\v' || c == '\f';
-    }
-
-    std::string trim(const std::string& str) {
-        if (str.size()) {
-            int start = 0;
-            for (size_t i = 0; i < str.length(); i++) {
-                if (!isSpace(str[i])) {
-                    break;
-                }
-                ++start;
-            }
-            int end = (int)str.length();
-            for (size_t i = str.length() - 1; i >= 0; i--) {
-                if (!isSpace(str[i])) {
-                    break;
-                }
-                --end;
-            }
-            if (end > start) {
-                std::string result = str.substr((size_t)start, (size_t)end - start);
-                return result;
-            }
-        }
-        return str;
-    }
-
-    static std::vector<std::string> split(
-        const std::string& in, const std::string& delim)
-    {
-        std::vector<std::string> result;
-        size_t last = 0, next = 0;
-        while ((next = in.find(delim, last)) != std::string::npos) {
-            result.push_back(std::move(trim(in.substr(last, next - last))));
-            last = next + 1;
-        }
-        result.push_back(std::move(trim(in.substr(last))));
-        return result;
-    }
-}
+static std::set<std::string> ID3V2_FORMATS = {
+    "mp3",
+    "aac"
+};
 
 #ifdef WIN32
 static inline std::wstring utf8to16(const char* utf8) {
@@ -196,12 +144,17 @@ static TagLib::FileRef resolveOggType(const char* uri) {
 }
 
 static bool isValidYear(const std::string& year) {
-    return std::stoi(year) > 0;
+    try {
+        return year.size() && std::stoi(year) > 0;
+    }
+    catch (...) {
+    }
+    return false;
 }
 
 static float toReplayGainFloat(const std::string& input) {
     /* trim any trailing " db" or "db" noise... */
-    std::string lower = str::lower(input);
+    std::string lower = str::ToLowerCopy(input);
     if (lower.find(" db") == lower.length() - 3) {
         lower = lower.substr(0, lower.length() - 3);
     }
@@ -229,6 +182,18 @@ static inline bool replayGainValid(ReplayGain& rg) {
         rg.trackGain != 1.0 || rg.trackPeak != 1.0;
 }
 
+static inline void processAlbumArt(TagLib::List<TagLib::FLAC::Picture*> pictures, ITagStore* target) {
+    for (auto picture : pictures) {
+        if (picture->type() == TagLib::FLAC::Picture::FrontCover) {
+            auto byteVector = picture->data();
+            if (byteVector.size()) {
+                target->SetThumbnail(byteVector.data(), byteVector.size());
+            }
+            break;
+        }
+    }
+}
+
 TaglibMetadataReader::TaglibMetadataReader() {
 }
 
@@ -241,27 +206,10 @@ void TaglibMetadataReader::Release() {
 
 bool TaglibMetadataReader::CanRead(const char *extension) {
     if (extension && strlen(extension)) {
-        std::string ext(str::lower(extension[0] == '.' ? &extension[1] : extension));
-        return
-#ifdef ENABLE_FFMPEG
-            ext.compare("opus") == 0 ||
-            ext.compare("wv") == 0 ||
-            ext.compare("wma") == 0 ||
-            ext.compare("ape") == 0 ||
-            ext.compare("mpc") == 0 ||
-            ext.compare("aac") == 0 ||
-            ext.compare("alac") == 0 ||
-            ext.compare("wav") == 0 ||
-            ext.compare("wave") == 0 ||
-            ext.compare("aif") == 0 ||
-            ext.compare("aiff") == 0 ||
-#endif
-            ext.compare("mp3") == 0 ||
-            ext.compare("ogg") == 0 ||
-            ext.compare("m4a") == 0 ||
-            ext.compare("flac") == 0;
+        std::string withoutLeadingDot = std::string(extension[0] == '.' ? &extension[1] : extension);
+        std::string ext = str::ToLowerCopy(withoutLeadingDot);
+        return SUPPORTED_FORMATS.find(ext) != SUPPORTED_FORMATS.end();
     }
-
     return false;
 }
 
@@ -269,7 +217,7 @@ bool TaglibMetadataReader::Read(const char* uri, ITagStore *track) {
     std::string path(uri);
     std::string extension;
 
-    std::string::size_type lastDot = path.find_last_of(".");
+    const std::string::size_type lastDot = path.find_last_of(".");
     if (lastDot != std::string::npos) {
         extension = path.substr(lastDot + 1).c_str();
     }
@@ -285,11 +233,16 @@ bool TaglibMetadataReader::Read(const char* uri, ITagStore *track) {
 
     /* ID3v2 is a trainwreck, so it requires special processing */
     if (extension.size()) {
-        if (str::lower(extension) == "mp3") {
+        if (ID3V2_FORMATS.find(str::ToLowerCopy(extension)) != ID3V2_FORMATS.end()) {
             this->ReadID3V2(uri, track);
         }
     }
 
+    /* always use the filename as the title if we can't resolve one */
+    if (!track->Contains("title")) {
+        this->SetTagValue("title", uri, track);
+    }
+
     return true;
 }
 
@@ -310,84 +263,83 @@ bool TaglibMetadataReader::ReadGeneric(
         file = resolveOggType(uri);
     }
 
-    if (file.isNull()) {
-        this->SetTagValue("title", uri, target);
-    }
-    else {
-        TagLib::Tag *tag = file.tag();
-        if (tag) {
-            this->ReadBasicData(file.tag(), uri, target);
+    TagLib::Tag *tag = file.tag();
+    if (tag) {
+        this->ReadBasicData(file.tag(), uri, target);
 
-            /* wav files can have metadata in the RIFF header, or, in some cases,
-            with an embedded id3v2 tag */
-            auto wavFile = dynamic_cast<TagLib::RIFF::WAV::File*>(file.file());
-            if (wavFile) {
-                if (wavFile->hasInfoTag()) {
-                    this->ReadBasicData(wavFile->InfoTag(), uri, target);
-                }
-                if (wavFile->hasID3v2Tag()) {
-                    this->ReadID3V2(wavFile->ID3v2Tag(), target);
-                }
+        /* wav files can have metadata in the RIFF header, or, in some cases,
+        with an embedded id3v2 tag */
+        auto wavFile = dynamic_cast<TagLib::RIFF::WAV::File*>(file.file());
+        if (wavFile) {
+            if (wavFile->hasInfoTag()) {
+                this->ReadBasicData(wavFile->InfoTag(), uri, target);
             }
-
-            /* aif files are similar to wav files, but for some reason taglib
-            doesn't seem to expose non-id3v2 tags */
-            auto aifFile = dynamic_cast<TagLib::RIFF::AIFF::File*>(file.file());
-            if (aifFile) {
-                if (aifFile->hasID3v2Tag()) {
-                    this->ReadID3V2(aifFile->tag(), target);
-                }
+            if (wavFile->hasID3v2Tag()) {
+                this->ReadID3V2(wavFile->ID3v2Tag(), target);
             }
+        }
 
-            /* taglib hides certain properties (like album artist) in the XiphComment's
-            field list. if we're dealing with a straight-up Xiph tag, process it now */
-            auto xiphTag = dynamic_cast<TagLib::Ogg::XiphComment*>(tag);
-            if (xiphTag) {
-                this->ReadFromMap(xiphTag->fieldListMap(), target);
-                this->ExtractReplayGain(xiphTag->fieldListMap(), target);
+        /* aif files are similar to wav files, but for some reason taglib
+        doesn't seem to expose non-id3v2 tags */
+        const auto aifFile = dynamic_cast<TagLib::RIFF::AIFF::File*>(file.file());
+        if (aifFile) {
+            if (aifFile->hasID3v2Tag()) {
+                this->ReadID3V2(aifFile->tag(), target);
             }
+        }
 
-            /* if this isn't a xiph tag, the file format may have some other custom
-            properties. let's see if we can pull them out here... */
-            if (!xiphTag) {
-                bool handled = false;
+        /* taglib hides certain properties (like album artist) in the XiphComment's
+        field list. if we're dealing with a straight-up Xiph tag, process it now */
+        const auto xiphTag = dynamic_cast<TagLib::Ogg::XiphComment*>(tag);
+        if (xiphTag) {
+            processAlbumArt(xiphTag->pictureList(), target);
+            this->ReadFromMap(xiphTag->fieldListMap(), target);
+            this->ExtractReplayGain(xiphTag->fieldListMap(), target);
+        }
 
-                /* flac files may have more than one type of tag embedded. see if there's
-                see if there's a xiph comment burried deep. */
-                auto flacFile = dynamic_cast<TagLib::FLAC::File*>(file.file());
-                if (flacFile && flacFile->hasXiphComment()) {
+        /* if this isn't a xiph tag, the file format may have some other custom
+        properties. let's see if we can pull them out here... */
+        if (!xiphTag) {
+            bool handled = false;
+
+            /* flac files may have more than one type of tag embedded. see if there's
+            see if there's a xiph comment buried deep. */
+            auto flacFile = dynamic_cast<TagLib::FLAC::File*>(file.file());
+            if (flacFile) {
+                processAlbumArt(flacFile->pictureList(), target);
+                if (flacFile->hasXiphComment()) {
                     this->ReadFromMap(flacFile->xiphComment()->fieldListMap(), target);
                     this->ExtractReplayGain(flacFile->xiphComment()->fieldListMap(), target);
                     handled = true;
                 }
+            }
 
-                /* similarly, mp4 buries disc number and album artist. however, taglib does
-                NOT exposed a map with normalized keys, so we have to do special property
-                handling here... */
-                if (!handled) {
-                    auto mp4File = dynamic_cast<TagLib::MP4::File*>(file.file());
-                    if (mp4File && mp4File->hasMP4Tag()) {
-                        auto mp4TagMap = static_cast<TagLib::MP4::Tag*>(tag)->itemListMap();
-                        this->ExtractValueForKey(mp4TagMap, "aART", "album_artist", target);
-                        this->ExtractValueForKey(mp4TagMap, "disk", "disc", target);
-                        this->ExtractReplayGain(mp4TagMap, target);
-                        handled = true;
-                    }
-                }
-
-                if (!handled) {
-                    auto wvFile = dynamic_cast<TagLib::WavPack::File*>(file.file());
-                    if (wvFile && wvFile->hasAPETag()) {
-                        this->ReadFromMap(wvFile->properties(), target);
-                        this->ExtractReplayGain(wvFile->properties(), target);
-                        handled = true;
-                    }
+            /* similarly, mp4 buries disc number and album artist. however, taglib does
+            NOT exposed a map with normalized keys, so we have to do special property
+            handling here... */
+            if (!handled) {
+                const auto mp4File = dynamic_cast<TagLib::MP4::File*>(file.file());
+                if (mp4File && mp4File->hasMP4Tag()) {
+                    auto mp4TagMap = static_cast<TagLib::MP4::Tag*>(tag)->itemListMap();
+                    this->ExtractValueForKey(mp4TagMap, "aART", "album_artist", target);
+                    this->ExtractValueForKey(mp4TagMap, "disk", "disc", target);
+                    this->ExtractReplayGain(mp4TagMap, target);
+                    handled = true;
                 }
             }
 
-            TagLib::AudioProperties *audio = file.audioProperties();
-            this->SetAudioProperties(audio, target);
+            if (!handled) {
+                const auto wvFile = dynamic_cast<TagLib::WavPack::File*>(file.file());
+                if (wvFile && wvFile->hasAPETag()) {
+                    this->ReadFromMap(wvFile->properties(), target);
+                    this->ExtractReplayGain(wvFile->properties(), target);
+                    handled = true;
+                }
+            }
         }
+
+        TagLib::AudioProperties *audio = file.audioProperties();
+        this->SetAudioProperties(audio, target);
     }
 
     return true;
@@ -415,7 +367,7 @@ std::string TaglibMetadataReader::ExtractValueForKey(
     if (map.contains(inputKey.c_str())) {
         TagLib::StringList value = map[inputKey.c_str()].toStringList();
         if (value.size()) {
-            return value[0].to8Bit();
+            return value[0].to8Bit(true);
         }
     }
     return defaultValue;
@@ -441,20 +393,15 @@ void TaglibMetadataReader::ReadFromMap(const T& map, ITagStore *target) {
     ExtractValueForKey(map, "DISCNUMBER", "disc", target);
     ExtractValueForKey(map, "ALBUM ARTIST", "album_artist", target);
     ExtractValueForKey(map, "ALBUMARTIST", "album_artist", target);
+    ExtractValueForKey(map, "RATING", "rating", target);
 }
 
 template<typename T>
 void TaglibMetadataReader::ReadBasicData(const T* tag, const char* uri, ITagStore *target) {
     if (tag) {
-        if (!tag->title().isEmpty()) {
-            this->SetTagValue("title", tag->title(), target);
-        }
-        else {
-            this->SetTagValue("title", uri, target);
-        }
-
+        this->SetTagValue("title", tag->title(), target);
         this->SetTagValue("album", tag->album(), target);
-        this->SetSlashSeparatedValues("artist", tag->artist(), target);
+        this->SetTagValue("artist", tag->artist(), target);
         this->SetTagValue("genre", tag->genre(), target);
         this->SetTagValue("comment", tag->comment(), target);
 
@@ -480,7 +427,7 @@ std::string TaglibMetadataReader::ExtractValueForKey(
     if (map.contains(inputKey.c_str())) {
         TagLib::StringList value = map[inputKey.c_str()];
         if (value.size()) {
-            return value[0].to8Bit();
+            return value[0].to8Bit(true);
         }
     }
     return defaultValue;
@@ -509,7 +456,7 @@ void TaglibMetadataReader::ExtractReplayGain(const T& map, ITagStore *target)
 void TaglibMetadataReader::SetTagValueWithPossibleTotal(
     const std::string& value, const std::string& valueKey, const std::string& totalKey, ITagStore* track)
 {
-    std::vector<std::string> parts = str::split(value, "/");
+    std::vector<std::string> parts = str::Split(value, "/");
     this->SetTagValue(valueKey.c_str(), parts[0].c_str(), track);
     if (parts.size() > 1) {
         this->SetTagValue(totalKey.c_str(), parts[1].c_str(), track);
@@ -554,21 +501,21 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
 
             if (!track->Contains("year") && !allTags["TYER"].isEmpty()) { /* ID3v2.3*/
                 auto year = allTags["TYER"].front()->toString().substr(0, 4);
-                if (isValidYear(year.to8Bit())) {
+                if (isValidYear(year.to8Bit(true))) {
                     this->SetTagValue("year", year, track);
                 }
             }
 
             if (!track->Contains("year") && !allTags["TDRC"].isEmpty()) { /* ID3v2.4*/
                 auto year = allTags["TDRC"].front()->toString().substr(0, 4);
-                if (isValidYear(year.to8Bit())) {
+                if (isValidYear(year.to8Bit(true))) {
                     this->SetTagValue("year", year, track);
                 }
             }
 
             if (!track->Contains("year") && !allTags["TCOP"].isEmpty()) { /* ID3v2.3*/
                 auto year = allTags["TCOP"].front()->toString().substr(0, 4);
-                if (isValidYear(year.to8Bit())) {
+                if (isValidYear(year.to8Bit(true))) {
                     this->SetTagValue("year", year, track);
                 }
             }
@@ -582,22 +529,22 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
 
                 for (auto current : txxx) {
                     using UTIF = TagLib::ID3v2::UserTextIdentificationFrame;
-                    UTIF* utif = dynamic_cast<UTIF*>(current);
+                    const UTIF* utif = dynamic_cast<UTIF*>(current);
                     if (utif) {
                         auto name = utif->description().upper();
                         auto values = utif->fieldList();
                         if (values.size() > 0) {
                             if (name == "REPLAYGAIN_TRACK_GAIN") {
-                                replayGain.trackGain = toReplayGainFloat(utif->fieldList().back().to8Bit());
+                                replayGain.trackGain = toReplayGainFloat(utif->fieldList().back().to8Bit(true));
                             }
                             else if (name == "REPLAYGAIN_TRACK_PEAK") {
-                                replayGain.trackPeak = toReplayGainFloat(utif->fieldList().back().to8Bit());
+                                replayGain.trackPeak = toReplayGainFloat(utif->fieldList().back().to8Bit(true));
                             }
                             else if (name == "REPLAYGAIN_ALBUM_GAIN") {
-                                replayGain.albumGain = toReplayGainFloat(utif->fieldList().back().to8Bit());
+                                replayGain.albumGain = toReplayGainFloat(utif->fieldList().back().to8Bit(true));
                             }
                             else if (name == "REPLAYGAIN_ALBUM_PEAK") {
-                                replayGain.albumPeak = toReplayGainFloat(utif->fieldList().back().to8Bit());
+                                replayGain.albumPeak = toReplayGainFloat(utif->fieldList().back().to8Bit(true));
                             }
                         }
                     }
@@ -610,13 +557,13 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
 
             /* TRCK is the track number (or "trackNum/totalTracks") */
             if (!allTags["TRCK"].isEmpty()) {
-                std::string trackNumber = allTags["TRCK"].front()->toString().toCString(true);
+                std::string trackNumber = allTags["TRCK"].front()->toString().to8Bit(true);
                 this->SetTagValueWithPossibleTotal(trackNumber, "track", "totaltracks", track);
             }
 
             /* TPOS is the disc number (or "discNum/totalDiscs") */
             if (!allTags["TPOS"].isEmpty()) {
-                std::string discNumber = allTags["TPOS"].front()->toString().toCString(true);
+                std::string discNumber = allTags["TPOS"].front()->toString().to8Bit(true);
                 this->SetTagValueWithPossibleTotal(discNumber, "disc", "totaldiscs", track);
             }
             else {
@@ -624,6 +571,9 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
                 this->SetTagValue("totaldiscs", "1", track);
             }
 
+            const int rating = this->ExtractRatingFromPopularimeter(allTags["POPM"]);
+            this->SetTagValue("rating", rating, track);
+
             this->SetTagValues("bpm", allTags["TBPM"], track);
             this->SetSlashSeparatedValues("composer", allTags["TCOM"], track);
             this->SetTagValues("copyright", allTags["TCOP"], track);
@@ -632,7 +582,7 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
             this->SetTagValues("org.writer", allTags["TOLY"], track);
             this->SetSlashSeparatedValues("publisher", allTags["TPUB"], track);
             this->SetTagValues("mood", allTags["TMOO"], track);
-            this->SetSlashSeparatedValues("org.artist", allTags["TOPE"], track);
+            this->SetTagValues("org.artist", allTags["TOPE"], track);
             this->SetTagValues("language", allTags["TLAN"], track);
             this->SetTagValues("lyrics", allTags["USLT"], track);
             this->SetTagValues("disc", allTags["TPOS"], track);
@@ -690,8 +640,8 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
 
             /* artists */
 
-            this->SetSlashSeparatedValues("artist", allTags["TPE1"], track);
-            this->SetSlashSeparatedValues("album_artist", allTags["TPE2"], track);
+            this->SetTagValues("artist", allTags["TPE1"], track);
+            this->SetTagValues("album_artist", allTags["TPE2"], track);
             this->SetSlashSeparatedValues("conductor", allTags["TPE3"], track);
             this->SetSlashSeparatedValues("interpreted", allTags["TPE4"], track);
 
@@ -701,7 +651,7 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
 
             TagLib::ID3v2::FrameList::Iterator it = comments.begin();
             for (; it != comments.end(); ++it) {
-                TagLib::ID3v2::CommentsFrame *comment
+                const TagLib::ID3v2::CommentsFrame *comment
                     = dynamic_cast<TagLib::ID3v2::CommentsFrame*> (*it);
 
                 TagLib::String temp = comment->description();
@@ -728,11 +678,11 @@ bool TaglibMetadataReader::ReadID3V2(TagLib::ID3v2::Tag *id3v2, ITagStore *track
                     /* there can be multiple pictures, apparently. let's just use
                     the first one. */
 
-                    TagLib::ID3v2::AttachedPictureFrame *picture =
+                    const TagLib::ID3v2::AttachedPictureFrame *picture =
                         static_cast<TagLib::ID3v2::AttachedPictureFrame*>(pictures.front());
 
                     TagLib::ByteVector pictureData = picture->picture();
-                    long long size = pictureData.size();
+                    const long long size = pictureData.size();
 
                     if (size > 32) {    /* noticed that some id3tags have like a 4-8 byte size with no thumbnail */
                         track->SetThumbnail(pictureData.data(), size);
@@ -780,7 +730,6 @@ void TaglibMetadataReader::SetTagValues(
 {
     if (!frame.isEmpty()) {
         TagLib::ID3v2::FrameList::ConstIterator value = frame.begin();
-
         for ( ; value != frame.end(); ++value) {
             TagLib::String tagString = (*value)->toString();
             if(!tagString.isEmpty()) {
@@ -794,9 +743,9 @@ void TaglibMetadataReader::SetTagValues(
 void TaglibMetadataReader::SetSlashSeparatedValues(
     const char* key, TagLib::String tagString, ITagStore *track)
 {
-    if(!tagString.isEmpty()) {
+    if (!tagString.isEmpty()) {
         std::string value(tagString.to8Bit(true));
-        std::vector<std::string> splitValues = str::split(value, "/");
+        std::vector<std::string> splitValues = str::Split(value, "/");
         std::vector<std::string>::iterator it = splitValues.begin();
         for( ; it != splitValues.end(); ++it) {
             track->SetValue(key, it->c_str());
@@ -809,7 +758,7 @@ void TaglibMetadataReader::SetSlashSeparatedValues(
     const TagLib::ID3v2::FrameList &frame,
     ITagStore *track)
 {
-    if(!frame.isEmpty()) {
+    if (!frame.isEmpty()) {
         TagLib::ID3v2::FrameList::ConstIterator value = frame.begin();
         for ( ; value != frame.end(); ++value) {
             TagLib::String tagString = (*value)->toString();
@@ -823,8 +772,8 @@ void TaglibMetadataReader::SetAudioProperties(
 {
     if (audioProperties) {
         std::string duration = std::to_string(audioProperties->length());
-        int bitrate = audioProperties->bitrate();
-        int channels = audioProperties->channels();
+        const int bitrate = audioProperties->bitrate();
+        const int channels = audioProperties->channels();
 
         this->SetTagValue("duration", duration, track);
 
@@ -837,3 +786,35 @@ void TaglibMetadataReader::SetAudioProperties(
         }
     }
 }
+
+int TaglibMetadataReader::ExtractRatingFromPopularimeter(const TagLib::ID3v2::FrameList& frame) {
+    /* the value of this tag is: 'some_str_identifier rating=[0-255] counter=[n]' */
+    if (!frame.isEmpty()) {
+        TagLib::ID3v2::FrameList::ConstIterator it = frame.begin();
+        for (; it != frame.end(); ++it) {
+            const TagLib::String rawTagValue = (*it)->toString();
+            if (!rawTagValue.isEmpty()) {
+                std::string utf8TagValue(rawTagValue.to8Bit(true));
+                const auto utf8TagValueParts = str::Split(utf8TagValue, " ");
+                if (utf8TagValueParts.size() > 2 && utf8TagValueParts.at(1).find_first_of("rating=") == 0) {
+                    const auto ratingParts = str::Split(utf8TagValueParts.at(1), "=");
+                    if (ratingParts.size() == 2) {
+                        const auto utf8Rating = ratingParts.at(1);
+                        try {
+                            const auto intRating = std::atoi(utf8Rating.c_str());
+                            if (intRating > 205) { return 5; }
+                            if (intRating > 154) { return 4; }
+                            if (intRating > 103) { return 3; }
+                            if (intRating > 52) { return 5; }
+                            if (intRating > 1) { return 1; }
+                        }
+                        catch (...) {
+                            /* invalid rating, couldn't be parsed as an int. */
+                        }
+                    }
+                }
+            }
+        }
+    }
+    return 0;
+}
